@@ -26,9 +26,10 @@ from sklearn.utils.testing import assert_less
 from sklearn.utils.testing import assert_raises
 from sklearn.utils.testing import assert_true
 from sklearn.utils.testing import assert_warns
+from sklearn.utils.testing import assert_warns_message
 from sklearn.utils.testing import skip_if_32bit
-from sklearn.utils.validation import DataConversionWarning
-from sklearn.utils.validation import NotFittedError
+from sklearn.exceptions import DataConversionWarning
+from sklearn.exceptions import NotFittedError
 
 # toy sample
 X = [[-2, -1], [-1, -1], [-1, -2], [1, 1], [1, 2], [2, 1]]
@@ -71,7 +72,8 @@ def check_classification_toy(presort, loss):
 
 
 def test_classification_toy():
-    for presort, loss in product(('auto', True, False), ('deviance', 'exponential')):
+    for presort, loss in product(('auto', True, False),
+                                 ('deviance', 'exponential')):
         yield check_classification_toy, presort, loss
 
 
@@ -95,11 +97,13 @@ def test_parameter_checks():
                   GradientBoostingClassifier(min_samples_split=0.0).fit, X, y)
     assert_raises(ValueError,
                   GradientBoostingClassifier(min_samples_split=-1.0).fit, X, y)
+    assert_raises(ValueError,
+                  GradientBoostingClassifier(min_samples_split=1.1).fit, X, y)
 
     assert_raises(ValueError,
                   GradientBoostingClassifier(min_samples_leaf=0).fit, X, y)
     assert_raises(ValueError,
-                  GradientBoostingClassifier(min_samples_leaf=-1.).fit, X, y)
+                  GradientBoostingClassifier(min_samples_leaf=-1.0).fit, X, y)
 
     assert_raises(ValueError,
                   GradientBoostingClassifier(min_weight_fraction_leaf=-1.).fit,
@@ -157,16 +161,15 @@ def check_classification_synthetic(presort, loss):
     X_train, X_test = X[:2000], X[2000:]
     y_train, y_test = y[:2000], y[2000:]
 
-    gbrt = GradientBoostingClassifier(n_estimators=100, min_samples_split=1,
+    gbrt = GradientBoostingClassifier(n_estimators=100, min_samples_split=2,
                                       max_depth=1, loss=loss,
-                                      learning_rate=1.0, random_state=0,
-                                      presort=presort)
+                                      learning_rate=1.0, random_state=0)
     gbrt.fit(X_train, y_train)
     error_rate = (1.0 - gbrt.score(X_test, y_test))
     assert_less(error_rate, 0.09)
 
-    gbrt = GradientBoostingClassifier(n_estimators=200, min_samples_split=1,
-                                      max_depth=1,
+    gbrt = GradientBoostingClassifier(n_estimators=200, min_samples_split=2,
+                                      max_depth=1, loss=loss,
                                       learning_rate=1.0, subsample=0.5,
                                       random_state=0,
                                       presort=presort)
@@ -185,12 +188,12 @@ def check_boston(presort, loss, subsample):
     # and least absolute deviation.
     ones = np.ones(len(boston.target))
     last_y_pred = None
-    for sample_weight in None, ones, 2*ones:
+    for sample_weight in None, ones, 2 * ones:
         clf = GradientBoostingRegressor(n_estimators=100,
                                         loss=loss,
                                         max_depth=4,
                                         subsample=subsample,
-                                        min_samples_split=1,
+                                        min_samples_split=2,
                                         random_state=1,
                                         presort=presort)
 
@@ -245,7 +248,7 @@ def test_regression_synthetic():
     # `Bagging Predictors?. Machine Learning 24(2): 123-140 (1996).
     random_state = check_random_state(1)
     regression_params = {'n_estimators': 100, 'max_depth': 4,
-                         'min_samples_split': 1, 'learning_rate': 0.1,
+                         'min_samples_split': 2, 'learning_rate': 0.1,
                          'loss': 'ls'}
 
     # Friedman1
@@ -292,20 +295,11 @@ def test_feature_importances():
 
     for presort in True, False:
         clf = GradientBoostingRegressor(n_estimators=100, max_depth=5,
-                                        min_samples_split=1, random_state=1,
+                                        min_samples_split=2, random_state=1,
                                         presort=presort)
         clf.fit(X, y)
         assert_true(hasattr(clf, 'feature_importances_'))
 
-        # XXX: Remove this test in 0.19 after transform support to estimators
-        # is removed.
-        X_new = assert_warns(
-            DeprecationWarning, clf.transform, X, threshold="mean")
-        assert_less(X_new.shape[1], X.shape[1])
-        feature_mask = (
-            clf.feature_importances_ > clf.feature_importances_.mean())
-        assert_array_almost_equal(X_new, X[:, feature_mask])
-
 
 def test_probability_log():
     # Predict probabilities.
@@ -952,7 +946,7 @@ def test_zero_estimator_clf():
 
 
 def test_max_leaf_nodes_max_depth():
-    # Test preceedence of max_leaf_nodes over max_depth.
+    # Test precedence of max_leaf_nodes over max_depth.
     X, y = datasets.make_hastie_10_2(n_samples=100, random_state=1)
     all_estimators = [GradientBoostingRegressor,
                       GradientBoostingClassifier]
@@ -968,6 +962,33 @@ def test_max_leaf_nodes_max_depth():
         assert_equal(tree.max_depth, 1)
 
 
+def test_min_impurity_split():
+    # Test if min_impurity_split of base estimators is set
+    # Regression test for #8006
+    X, y = datasets.make_hastie_10_2(n_samples=100, random_state=1)
+    all_estimators = [GradientBoostingRegressor, GradientBoostingClassifier]
+
+    for GBEstimator in all_estimators:
+        est = GBEstimator(min_impurity_split=0.1)
+        est = assert_warns_message(DeprecationWarning, "min_impurity_decrease",
+                                   est.fit, X, y)
+        for tree in est.estimators_.flat:
+            assert_equal(tree.min_impurity_split, 0.1)
+
+
+def test_min_impurity_decrease():
+    X, y = datasets.make_hastie_10_2(n_samples=100, random_state=1)
+    all_estimators = [GradientBoostingRegressor, GradientBoostingClassifier]
+
+    for GBEstimator in all_estimators:
+        est = GBEstimator(min_impurity_decrease=0.1)
+        est.fit(X, y)
+        for tree in est.estimators_.flat:
+            # Simply check if the parameter is passed on correctly. Tree tests
+            # will suffice for the actual working of this param
+            assert_equal(tree.min_impurity_decrease, 0.1)
+
+
 def test_warm_start_wo_nestimators_change():
     # Test if warm_start does nothing if n_estimators is not changed.
     # Regression test for #3513.
@@ -1010,7 +1031,8 @@ def test_non_uniform_weights_toy_edge_case_reg():
     # ignore the first 2 training samples by setting their weight to 0
     sample_weight = [0, 0, 1, 1]
     for loss in ('huber', 'ls', 'lad', 'quantile'):
-        gb = GradientBoostingRegressor(learning_rate=1.0, n_estimators=2, loss=loss)
+        gb = GradientBoostingRegressor(learning_rate=1.0, n_estimators=2,
+                                       loss=loss)
         gb.fit(X, y, sample_weight=sample_weight)
         assert_greater(gb.predict([[1, 0]])[0], 0.5)
 
@@ -1024,13 +1046,14 @@ def test_non_uniform_weights_toy_edge_case_clf():
     # ignore the first 2 training samples by setting their weight to 0
     sample_weight = [0, 0, 1, 1]
     for loss in ('deviance', 'exponential'):
-        gb = GradientBoostingClassifier(n_estimators=5)
+        gb = GradientBoostingClassifier(n_estimators=5, loss=loss)
         gb.fit(X, y, sample_weight=sample_weight)
         assert_array_equal(gb.predict([[1, 0]]), [1])
 
 
 def check_sparse_input(EstimatorClass, X, X_sparse, y):
-    dense = EstimatorClass(n_estimators=10, random_state=0, max_depth=2).fit(X, y)
+    dense = EstimatorClass(n_estimators=10, random_state=0,
+                           max_depth=2).fit(X, y)
     sparse = EstimatorClass(n_estimators=10, random_state=0, max_depth=2,
                             presort=False).fit(X_sparse, y)
     auto = EstimatorClass(n_estimators=10, random_state=0, max_depth=2,
@@ -1046,6 +1069,9 @@ def check_sparse_input(EstimatorClass, X, X_sparse, y):
     assert_array_almost_equal(sparse.feature_importances_,
                               auto.feature_importances_)
 
+    assert_array_almost_equal(sparse.predict(X_sparse), dense.predict(X))
+    assert_array_almost_equal(dense.predict(X_sparse), sparse.predict(X))
+
     if isinstance(EstimatorClass, GradientBoostingClassifier):
         assert_array_almost_equal(sparse.predict_proba(X),
                                   dense.predict_proba(X))
@@ -1057,6 +1083,15 @@ def check_sparse_input(EstimatorClass, X, X_sparse, y):
         assert_array_almost_equal(sparse.predict_log_proba(X),
                                   auto.predict_log_proba(X))
 
+        assert_array_almost_equal(sparse.decision_function(X_sparse),
+                                  sparse.decision_function(X))
+        assert_array_almost_equal(dense.decision_function(X_sparse),
+                                  sparse.decision_function(X))
+
+        assert_array_almost_equal(
+            np.array(sparse.staged_decision_function(X_sparse)),
+            np.array(sparse.staged_decision_function(X)))
+
 
 @skip_if_32bit
 def test_sparse_input():
